from dataclasses import dataclass
from datetime import datetime
from typing import Optional
from pytest_bdd_report.interfaces import ISummary

@dataclass
class Summary(ISummary):
    report_title: str = ""
    total_tests: int = 0
    tests_passed: int = 0
    tests_failed: int = 0
    tests_skipped: int = 0
    percentage_tests_passed: float = 0.0
    start_time: Optional[datetime] = None
    total_duration: float = 0
    top_feature_fail: list = None



from pytest_bdd_report.components.scenario import Scenario
from dataclasses import dataclass


@dataclass
class Feature:
    def __init__(
        self,
        id: str,
        name: str,
        line: int,
        description: str,
        tags: list,
        uri: str,
        scenarios: list[Scenario],
        duration: float = 0.0,
        status: str = "passed",
    ):
        self.id = id
        self.name = name
        self.line = line
        self.description = description
        self.tags = tags
        self.uri = uri
        self.scenarios = scenarios
        self.total_tests = 0
        self.passed_tests = 0
        self.failed_tests = 0
        self.skipped_tests = 0
        self.duration = duration
        self.status = status

    def calculate_duration(self) -> None:
        duration = 0
        for scenario in self.scenarios:
            duration += scenario.duration
        self.duration = duration

    def add_scenario(self, scenario: Scenario) -> None:
        self.scenarios.append(scenario)

    def set_total_tests(self, quantity: int):
        self.total_tests = quantity

    def set_failed_tests(self, quantity: int):
        self.failed_tests = quantity

    def set_passed_tests(self, quantity: int):
        self.passed_tests = quantity

    def set_skipped_test(self, quantity: int):
        self.skipped_tests = quantity

    def __eq__(self, other):
        if isinstance(other, Feature):
            return (
                self.id == other.id
                and self.name == other.name
                and self.description == other.description
                and self.tags == other.tags
                and self.line == other.line
                and self.uri == other.uri
                and self.duration == other.duration
                and self.status == other.status
                and self.total_tests == other.total_tests
                and self.passed_tests == other.passed_tests
                and self.failed_tests == other.failed_tests
                and self.skipped_tests == other.skipped_tests
                and self.scenarios == other.scenarios
            )
        return False



from pytest_bdd_report.components.step import Step
from pytest_bdd_report.templates.template import BaseTemplate


class StepTemplate(BaseTemplate):
    def __init__(self) -> None:
        self.path = "step.html"
        super().__init__(self.path)

    def render_template(self, data: Step, **kwargs) -> str:
        return self.template.render(
            keyword=data.keyword,
            name=data.name,
            status=data.status,
            duration=data.duration,
        )



from typing import List
from dataclasses import dataclass
from pytest_bdd_report.components.step import Step


@dataclass
class Scenario:
    id: str
    name: str
    line: int
    description: str
    tags: List[str]
    steps: List[Step]
    duration: float = 0.0
    status: str = "passed"
    error_message: str = ""

    def calculate_duration(self) -> None:
        self.duration = sum(step.duration for step in self.steps) / 1_000_000_000  # from nanosecond to second

    def add_step(self, step: Step) -> None:
        self.steps.append(step)

    def check_and_add_error_message(self):
        for step in self.steps:
            if step.error_message:
                self.error_message = step.error_message
                break



from abc import ABC, abstractmethod

from pytest_bdd_report.components.feature import Feature
from pytest_bdd_report.components.scenario import Scenario
from pytest_bdd_report.components.step import Step
from pytest_bdd_report.templates.feature_statistics_template import (
    FeatureStatisticsTemplate,
)
from pytest_bdd_report.templates.feature_template import FeatureTemplate
from pytest_bdd_report.templates.scenario_template import ScenarioTemplate
from pytest_bdd_report.templates.step_template import StepTemplate
from pytest_bdd_report.templates.template import BaseTemplate


class BaseRenderer(ABC):
    @abstractmethod
    def render(self, items: list, template: BaseTemplate) -> str:
        """
        Render the items into the template.
        @param items: to render
        @param template: in which render the items
        @return: rendered object
        """
        ...


class StepRenderer(BaseRenderer):
    def render(self, items: list[Step], template: BaseTemplate) -> str:
        return ''.join(template.render_template(item) for item in items)


class ScenarioRenderer(BaseRenderer):
    def __init__(self, step_renderer: StepRenderer = None):
        self.step_renderer = step_renderer or StepRenderer()

    def render(self, items: list[Scenario], template: BaseTemplate) -> str:
        rendered_steps = [self.step_renderer.render(item.steps, StepTemplate()) for item in items]
        return ''.join(template.render_template(item, steps) for item, steps in zip(items, rendered_steps))


class FeatureRenderer(BaseRenderer):
    def __init__(self, scenario_renderer: ScenarioRenderer = None):
        self.scenario_renderer = scenario_renderer or ScenarioRenderer()

    def render(self, items: list[Feature], template: BaseTemplate) -> str:
        rendered_scenarios = [self.scenario_renderer.render(item.scenarios, ScenarioTemplate()) for item in items]
        return ''.join(template.render_template(item, scenarios) for item, scenarios in zip(items, rendered_scenarios))



class FeatureStatisticsRenderer(BaseRenderer):
    def render(self, items: list[Feature], template: BaseTemplate) -> str:
        return ''.join(template.render_template(item) for item in items)



from pytest_bdd_report.interfaces import IReport
from pytest_bdd_report.summary.summary import Summary
from typing import List


class SummaryGenerator:
    def __init__(self):
        self.summary = Summary()

    def populate_summary(self, report: IReport) -> Summary:
        """
        Populate the summary based on the report provided.
        @param report: from which create the summary
        @return: summary object
        """
        self._set_report_title(report)
        self._get_test_statistics(report)
        self._get_top_feature_fail(report)
        self._get_total_duration(report)
        self._calculate_percentage_test_passed()
        return self.summary

    def _set_report_title(self, report: IReport) -> None:
        """
        Set the summary title as the report title.
        @param report:
        @return:
        """
        self.summary.report_title = report.title

    def _get_test_statistics(self, report: IReport) -> None:
        """
        Calculate the test session statistics based on the report provided.
        @param report:
        @return:
        """
        for feature in report.features:
            self.summary.total_tests += feature.total_tests
            self.summary.tests_passed += feature.passed_tests
            self.summary.tests_failed += feature.failed_tests
            self.summary.tests_skipped += feature.skipped_tests

    def _get_top_feature_fail(self, report: IReport) -> None:
        """
        Get the top 5 features with the highest number of failed tests.
        @param report:
        @return:
        """
        failed_features = [feature for feature in report.features if feature.failed_tests > 0]
        sorted_features = sorted(failed_features, key=lambda x: x.failed_tests, reverse=True)
        self.summary.top_feature_fail = sorted_features[:5]

    def _get_total_duration(self, report: IReport) -> None:
        """
        Calculate the total duration of the test session based on the report provided.
        @param report:
        @return:
        """
        self.summary.total_duration = sum(feature.duration for feature in report.features)

    def _calculate_percentage_test_passed(self) -> None:
        """
        Calculate the percentage of the test passed based on the total number of tests.
        @return:
        """
        if self.summary.total_tests != 0:
            self.summary.percentage_tests_passed = round(
                self.summary.tests_passed / self.summary.total_tests * 100, 0
            )



from abc import ABC, abstractmethod
from typing import Protocol
from pytest_bdd_report.components.feature import Feature


class IReport(Protocol):
    title: str
    features: list[Feature]


class ISummary(Protocol):
    total_test: int
    test_passed: int
    test_failed: int
    test_skipped: int


class ILoader(Protocol):
    path: str  # For type checking, indicating implementing classes should have this attribute

    def load(self) -> list[dict]:
        ...


class IReportBuilder(Protocol):
    def set_features(self, features: list):
        ...

    def build(self) -> IReport:
        """
        Build the report, this should be used at the end of the building function chain.
        @return: report object
        """
        ...



import json
import os

from pytest_bdd_report.interfaces import ILoader


class JsonLoader(ILoader):
    def __init__(self, path: str):
        self.path = path

    def load(self) -> list[dict]:
        if os.path.exists(self.path):
            with open(self.path, "r", encoding="utf-8") as f:
                return json.load(f)
        else:
            return []



from pytest_bdd_report.components.feature import Feature
from pytest_bdd_report.templates.template import BaseTemplate


class FeatureStatisticsTemplate(BaseTemplate):
    def __init__(self) -> None:
        self.path = "feature_statistics.html"
        super().__init__(self.path)

    def render_template(self, data: Feature, already_rendered_data: str = "") -> str:
        passed_rate = self._calculate_passed_rate(data)
        return self.template.render(
            name=data.name,
            total=data.total_tests,
            passed=data.passed_tests,
            failed=data.failed_tests,
            skipped=data.skipped_tests,
            duration=round(data.duration, 5),
            rate=passed_rate,
        )

    def _calculate_passed_rate(self, data: Feature) -> int:
        """
        Calculate the passed rate percentage.
        """
        if data.total_tests == 0:
            return 0
        return int(round(data.passed_tests / data.total_tests * 100))



from pytest_bdd_report.components.feature import Feature
from pytest_bdd_report.templates.template import BaseTemplate
from typing import List, Dict


class FeatureTemplate(BaseTemplate):
    def __init__(self) -> None:
        self.path = "feature.html"
        super().__init__(self.path)

    def render_template(self, data: Feature, rendered_scenarios: str = "") -> str:
        return self.template.render(
            id=data.id,
            name=data.name,
            status=data.status,
            duration=data.duration,
            scenarios=rendered_scenarios,
            total=data.total_tests,
            passed=data.passed_tests,
            failed=data.failed_tests,
            skipped=data.skipped_tests,
            description=data.description,
            tags=self._format_tags(data.tags),
        )

    @staticmethod
    def _format_tags(tags: List[Dict[str, str]]) -> str:
        """
        Format tags into a comma-separated string.
        """
        if not tags:
            return ""
        return ", ".join(tag["name"] for tag in tags)



import os.path

from pytest_bdd_report.interfaces import IReport, ISummary
from pytest_bdd_report.renderer import FeatureRenderer, FeatureStatisticsRenderer
from pytest_bdd_report.summary.summary import Summary
from pytest_bdd_report.templates.feature_statistics_template import (
    FeatureStatisticsTemplate,
)
from pytest_bdd_report.templates.feature_template import FeatureTemplate
from pytest_bdd_report.templates.report_template import ReportTemplate
from pytest_bdd_report.templates.summary_template import SummaryTemplate


class ReportFileGenerator:
    def __init__(self) -> None:
        self.report_content = ""

    def create_report_file(
        self, report: IReport, summary: ISummary, test_file_uri: list[str], path: str
    ) -> None:
        """
        Create the report in the provided file path.
        """
        rendered_features = self._render_features(report)
        rendered_summary = self._render_summary(summary)
        rendered_feature_statistics = self._render_feature_statistics(report)
        self.report_content = self._render_report(
            report.title,
            test_file_uri,
            str(os.path.abspath(path)),
            rendered_summary,
            rendered_features,
            rendered_feature_statistics,
        )
        self._save_report_to_file(path)

    @staticmethod
    def _render_features(report: IReport) -> str:
        """
        Render the features and return them as a string.
        """
        feature_renderer = FeatureRenderer()
        return feature_renderer.render(report.features, FeatureTemplate())

    @staticmethod
    def _render_summary(data: ISummary) -> str:
        """
        Render the summary and return it as a string.
        """
        summary_template = SummaryTemplate()
        return summary_template.render_template(data)

    @staticmethod
    def _render_feature_statistics(report: IReport) -> str:
        feature_statistics_renderer = FeatureStatisticsRenderer()
        return feature_statistics_renderer.render(
            report.features, FeatureStatisticsTemplate()
        )

    @staticmethod
    def _render_report(
        title: str,
        test_file_uri: list[str],
        file_path: str,
        rendered_summary: str,
        rendered_features: str,
        rendered_feature_statistics: str,
    ) -> str:
        report_template = ReportTemplate()
        report_template.add_rendered_summary(rendered_summary)
        report_template.add_rendered_features(rendered_features)
        report_template.add_rendered_feature_statistics(rendered_feature_statistics)
        report_template.add_test_file_uri(test_file_uri)
        report_template.add_file_path(file_path)
        return report_template.render_template(title)

    def _save_report_to_file(self, path: str) -> None:
        """
        Save the report content to the provided file path.
        If the path not exists it will be created.
        """
        if "/" in path:
            os.makedirs(os.path.dirname(path), exist_ok=True)

        with open(path, "w", encoding="utf-8") as f:
            f.write(self.report_content)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import codecs
from setuptools import setup


def read(fname):
    file_path = os.path.join(os.path.dirname(__file__), fname)
    return codecs.open(file_path, encoding="utf-8").read()


setup(
    name="pytest-bdd-report",
    version="0.2.0",
    author="Mattia Monti",
    author_email="mattiamonti2001@gmail.com",
    maintainer="Mattia Monti",
    maintainer_email="mattiamonti2001@gmail.com",
    license="MIT",
    url="https://github.com/mattiamonti/pytest-bdd-report",
    description="A pytest-bdd plugin for generating useful and informative BDD test reports",
    long_description=read("README.rst"),
    py_modules=[
        "pytest_bdd_report",
    ],
    python_requires=">=3.5",
    install_requires=["pytest>=3.5.0", "pytest-bdd"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Framework :: Pytest",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Testing",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
    ],
    entry_points={
        "pytest11": [
            "bdd-report = pytest_bdd_report",
        ],
    },
)



import re
from pytest_bdd_report.components.scenario import Scenario
from pytest_bdd_report.templates.template import BaseTemplate
from typing import List, Dict


class ScenarioTemplate(BaseTemplate):
    def __init__(self) -> None:
        self.path: str = "scenario.html"
        super().__init__(self.path)

    def render_template(self, data: Scenario, rendered_steps: str = "") -> str:
        """
        Render the scenario template.
        """
        return self.template.render(
            id=data.id,
            name=data.name,
            status=data.status,
            duration=data.duration,
            steps=rendered_steps,
            error_message=data.error_message,
            description=data.description,
            tags=self._format_tags(data.tags),
            parameters=self._check_for_parameters(data.id),
        )

    @staticmethod
    def _format_tags(tags: List[Dict[str, str]]) -> str:
        """
        Format tags into a comma-separated string.
        """
        return ", ".join(tag["name"] for tag in tags)

    @staticmethod
    def _check_for_parameters(id: str) -> str:
        """
        Check for parameters in the scenario ID and format them.
        """
        match = re.search(r"\[(.*?)]", id)
        if match:
            return match.group(1).replace("-", ", ")
        return ""



import datetime
from typing import List

from pytest_bdd_report.templates.template import BaseTemplate


class ReportTemplate(BaseTemplate):
    def __init__(self) -> None:
        self.rendered_summary: str = ""
        self.rendered_features: str = ""
        self.rendered_feature_statistics: str = ""
        self.test_file_uri: List[str] = []
        self.path: str = "report.html"
        self.file_path: str = ""
        super().__init__(self.path)

    def render_template(self, data: str, **kwargs) -> str:
        """
        Render the report template.
        """
        return self.template.render(
            title=data,
            date_time=datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
            test_file_uri=self.test_file_uri,
            file_path=self.file_path,
            summary=self.rendered_summary,
            features=self.rendered_features,
            feature_statistics=self.rendered_feature_statistics,
        )

    def add_rendered_summary(self, rendered_summary: str) -> None:
        """
        Add rendered summary to inject into the template.
        """
        self.rendered_summary = rendered_summary

    def add_rendered_features(self, rendered_features: str) -> None:
        """
        Add rendered features to inject into the template.
        """
        self.rendered_features = rendered_features

    def add_rendered_feature_statistics(self, rendered_feature_statistics: str) -> None:
        """
        Add rendered feature statistics to inject into the template.
        """
        self.rendered_feature_statistics = rendered_feature_statistics

    def add_test_file_uri(self, test_file_uri: List[str]) -> None:
        """
        Add test file URIs to the template.
        """
        self.test_file_uri = test_file_uri

    def add_file_path(self, file_path: str) -> None:
        """
        Add file path to the template.
        """
        self.file_path = file_path



from dataclasses import dataclass
from pytest_bdd_report.components.feature import Feature
from pytest_bdd_report.interfaces import IReportBuilder


@dataclass
class Report:
    title: str
    features: list[Feature]

    def add_feature(self, feature: Feature) -> None:
        self.features.append(feature)


class ReportBuilder(IReportBuilder):
    def __init__(self, title: str):
        self.report = Report(title, [])

    def set_features(self, features: list[Feature]):
        self.report.features = features
        return self

    def build(self) -> Report:
        return self.report



from pathlib import Path
from abc import ABC, abstractmethod
from jinja2 import Environment, FileSystemLoader, PackageLoader


class BaseTemplate(ABC):
    def __init__(self, path: str) -> None:
        self.path = path
        self.template = self._load_template()

    def _load_template(self):
        """
        Load the template from the html file.
        @return: template
        """
        resources_path = Path(__file__).parent.joinpath("html_templates")
        environment = Environment(loader=FileSystemLoader([resources_path]))
        return environment.get_template(self.path)

    @abstractmethod
    def render_template(self, data, already_rendered_data: str = "") -> str:
        """
        Render the template with the data provided.
        @param data: object to render
        @param already_rendered_data: (optional) rendered object to inject as plain html
        @return: rendered object
        """
        ...



from pytest_bdd_report.extractor import FeatureExtractor
from pytest_bdd_report.interfaces import IReport, ILoader, IReportBuilder


class ReportComposer:
    def __init__(self, loader: ILoader, report_builder: IReportBuilder) -> None:
        self.data: list[dict] = loader.load()
        self.report_builder = report_builder
        self.report = None

    def create_report(self) -> IReport:
        """
        Generate the report for the data loaded in the constructor.
        @return: the report
        """
        features = self._extract_features()
        self.report_builder.set_features(features)
        self.report = self.report_builder.build()
        self._calculate_durations()
        return self.report

    def _extract_features(self) -> list:
        """
        Extract features from the object data.
        @return: features
        """
        return FeatureExtractor().extract_from(self.data)

    def _calculate_durations(self) -> None:
        """
        Calculate the duration for the scenarios and the features.
        """
        for feature in self.report.features:
            for scenario in feature.scenarios:
                scenario.calculate_duration()
            feature.calculate_duration()



from pytest_bdd_report.summary.summary import Summary
from pytest_bdd_report.templates.template import BaseTemplate


class SummaryTemplate(BaseTemplate):
    def __init__(self) -> None:
        self.path = "summary.html"
        super().__init__(self.path)

    def render_template(self, data: Summary, **kwargs) -> str:
        return self.template.render(
            total=data.total_tests,
            passed=data.tests_passed,
            failed=data.tests_failed,
            skipped=data.tests_skipped,
            percentage=data.percentage_tests_passed,
            duration=round(data.total_duration, 5),
            top_feature_fail=data.top_feature_fail,
        )



import os
from abc import ABC, abstractmethod
from typing import List

from pytest_bdd_report.components.feature import Feature
from pytest_bdd_report.components.scenario import Scenario
from pytest_bdd_report.components.step import Step


class BaseExtractor(ABC):
    @abstractmethod
    def extract_from(self, data: List[dict]) -> List:
        """
        Extract the objects from the raw data passed.
        """
        pass

    @abstractmethod
    def create_item(self, data: dict):
        """
        Create one item from the data passed.
        """
        pass


class StepExtractor(BaseExtractor):
    def create_item(self, data: dict) -> Step:
        step = Step(
            keyword=data["keyword"],
            name=data["name"],
            line=data["line"],
            status=data["result"]["status"],
            duration=data["result"]["duration"],
        )
        step.error_message = data["result"].get("error_message", "")
        return step

    def extract_from(self, data: List[dict]) -> List[Step]:
        return [self.create_item(item_data) for item_data in data]


class ScenarioExtractor(BaseExtractor):
    def create_item(self, data: dict) -> Scenario:
        steps = StepExtractor().extract_from(data.get("steps", []))
        status = "failed" if any(step.status == "failed" for step in steps) else "passed"
        scenario = Scenario(
            id=data["id"],
            name=data["name"],
            line=data["line"],
            description=data["description"],
            tags=data["tags"],
            steps=steps,
            status=status,
        )
        scenario.check_and_add_error_message()
        return scenario

    def extract_from(self, data: List[dict]) -> List[Scenario]:
        return [self.create_item(item_data) for item_data in data]


class FeatureExtractor(BaseExtractor):
    def create_item(self, data: dict) -> Feature:
        scenarios = ScenarioExtractor().extract_from(data.get("elements", []))
        failed, passed, total = self._count_failed_passed_total_tests(scenarios)
        status = "passed" if failed == 0 else "failed"
        feature = Feature(
            id=data["id"],
            name=data["name"],
            uri=data["uri"],
            line=data["line"],
            description=data["description"],
            tags=data["tags"],
            scenarios=scenarios,
            status=status,
        )
        skipped = self._check_for_skipped_scenarios(feature)
        feature.set_total_tests(total + skipped)
        feature.set_failed_tests(failed)
        feature.set_passed_tests(passed)
        feature.set_skipped_test(skipped)
        return feature

    def extract_from(self, data: List[dict]) -> List[Feature]:
        return [self.create_item(item_data) for item_data in data]

    @staticmethod
    def _count_failed_passed_total_tests(tests: List[Scenario]) -> tuple[int, int, int]:
        failed = sum(1 for test in tests if test.status == "failed")
        passed = sum(1 for test in tests if test.status == "passed")
        total = len(tests)
        return failed, passed, total

    @staticmethod
    def _check_for_skipped_scenarios(feature: Feature) -> int:
        try:
            with open(os.path.abspath(feature.uri), "r", encoding="utf-8") as f:
                lines = f.readlines()

            if not lines:
                return 0

            scenario_file_names = [line.split(":")[1].strip() for line in lines if line.strip().startswith("Scenario")]
            scenario_names = [scenario.name for scenario in feature.scenarios]

            skipped = sum(1 for name in scenario_file_names if name not in scenario_names)
            for name in scenario_file_names:
                if name not in scenario_names:
                    feature.add_scenario(
                        Scenario(
                            id="",
                            name=name,
                            line=0,
                            description="",
                            tags=[],
                            steps=[],
                            status="skipped",
                        )
                    )
            return skipped
        except:
            return 0



import pytest
from pytest_bdd_report.json_loader import JsonLoader
from pytest_bdd_report.report_composer import ReportComposer
from pytest_bdd_report.report import Report, ReportBuilder
from pytest_bdd_report.report_file_generator import ReportFileGenerator
from pytest_bdd_report.summary.summary_generator import SummaryGenerator
import os

BDD_REPORT_FLAG = "--bdd-report"

# the default path for the cucumber-json file
# (it will be used if session.config.option.cucumber_json_path is not set)
DEFAULT_CUCUMBER_JSON_PATH = ".cucumber-data.json"

# Command-line option setup
def pytest_addoption(parser):
    """
    Add the command line option to generate the HTML report
    """
    group = parser.getgroup("bdd-report")
    group.addoption(
        BDD_REPORT_FLAG,
        action="store",
        default="",
        help="Create the BDD tests report at the given path.",
    )


# Command-line option getter
def _get_flag_option(config, flag: str) -> str:
    """
    @return the option value
    """
    return config.getoption(flag).replace(".html", "") + ".html"


def pytest_configure(config):
    """
    Configure the generation of the cucumber-json file
    """
    # if the flag option is not set, do nothing
    if _get_flag_option(config, BDD_REPORT_FLAG) == ".html":
        return
    
    # Set the cucumber-json path with a default value if not already set
    if not config.option.cucumber_json_path:
        config.option.cucumber_json_path = DEFAULT_CUCUMBER_JSON_PATH
        
test_file_uri = []


def pytest_collection_modifyitems(config, items):
    if _get_flag_option(config, BDD_REPORT_FLAG) != ".html":
        for item in items:
            uri = item.nodeid.split("::")[0]
            if uri not in test_file_uri:
                test_file_uri.append(uri)


def pytest_terminal_summary(terminalreporter, exitstatus, config):
    """
    Log the report file path in the teminal
    """
    bdd_report_flag = _get_flag_option(config, BDD_REPORT_FLAG)
    if bdd_report_flag != ".html":
        print(f"\n\n Report created at: {bdd_report_flag}")


@pytest.hookimpl(trylast=True)
def pytest_sessionfinish(session):
    """
    Run the plugin logic after the entire test session execution
    """
    report_file_path = _get_flag_option(session.config, BDD_REPORT_FLAG)
    if report_file_path != ".html":
        report_name = os.path.basename(report_file_path)
        report_generator = ReportComposer(
            loader=JsonLoader(session.config.option.cucumber_json_path),
            report_builder=ReportBuilder(report_name),
        )
        report = report_generator.create_report()
        summary = SummaryGenerator().populate_summary(report)
        file_generator = ReportFileGenerator()
        file_generator.create_report_file(
            report, summary, test_file_uri, report_file_path
        )
        print(test_file_uri)
